// data_file is from endpointer
// audio_file is JEIDA data
// out_file changes, and is the raw audio data of JEDIA utterances
// parameter_file is the same parameter_file endpointer uses

// system include files

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// isip include files

#include <math/integral/integral.h>
#include <math/integral/integral_constants.h>

// local include files

#include "make_files.h"

main(int argc, char *argv[])
{

  FILE *data_file, *audio_file, *parameter_file, *output_file;
  float beg_time, end_time, utterance_duration;
  short int *channel_0, *channel_1, *utterance_ptr, *channel_0_front, *channel_1_front, *utterance_ptr_front;
  int number_of_channels=0, counter=0, sample_frequency, utterance_number;
  long unsigned int num_samples=0, samples_read=0;
  char endpointer_data_name[80] = DEFAULT_ENDPOINTER_DATA_NAME,
       param_file_name[80] = DEFAULT_PARAM_FILE_NAME,
       output_dir[80] = DEFAULT_OUTPUT_DIR, 
       base_name[80] = DEFAULT_BASE_NAME, 
       prefix[80] = DEFAULT_PREFIX, 
       extension[80] = DEFAULT_EXTENSION, 
       output_file_name[80] = DEFAULT_OUTPUT_FILE_NAME,
       data_file_string[80] = DEFAULT_DATA_FILE_STRING,
       audio_file_name[80] = DEFAULT_AUDIO_FILE_NAME;


  if(!get_command_line(argc, argv, param_file_name, output_dir, base_name, 
		       prefix, extension, audio_file_name, 
		       endpointer_data_name))
    {
      fprintf(stderr, "%s: error in command line arguments!  Use make_files.exe -help for help\n", ISIP_PROGRAM_NAME);
       exit(ISIP_FATAL_ERROR);
      
    }

  // open and test each of the files
  
  parameter_file = fopen(param_file_name, "r");
  if(!parameter_file)
    {
      fprintf(stderr, "Parameter file (%s) could not be opened!\n", param_file_name);
      exit(ISIP_FATAL_ERROR);
    }

  data_file = fopen(endpointer_data_name, "r");
  if(!data_file)
    {
      fprintf(stderr, "Endpoints data file (%s) could not be opened!\n", endpointer_data_name);
      exit(ISIP_FATAL_ERROR);
    }

  audio_file = fopen(audio_file_name, "r");
  if(!audio_file)
    {
      fprintf(stderr, "Audio file (%s) could not be opened!\n", audio_file_name);
      exit(ISIP_FATAL_ERROR);
    }

  // get parameters from parameter_file
  
  number_of_channels = (int)get_parameters("number_of_channels", parameter_file);
  sample_frequency = (int)get_parameters("sample_frequency", parameter_file);
  fclose(parameter_file);
  
  // read in an utterance position from data_file, the coorsponding data from the audio_file,
  // and write the raw audio data to disk.

  while((fgets(data_file_string, sizeof(data_file_string), data_file)))
    {      
      sscanf(data_file_string, SSCANF_FORMAT, &utterance_number, &beg_time, &end_time);

      // move to that position in the file
      
      fseek(audio_file, (long int)(beg_time * sample_frequency * SAMPLE_SIZE * number_of_channels), SEEK_SET);
      
      // calculate the utterance duration and how many samples that duration will have
      
      utterance_duration = (end_time) - (beg_time);
      if(utterance_duration <= 0) 
	{

	  fprintf(stderr, "Illogical utterance duration found: %5.3f.  Cooresponding utterance file will be of zero length.\n", utterance_duration);
	  utterance_duration = 0;
	}
      num_samples = (long unsigned int)((utterance_duration)*(sample_frequency));

      // read in the whole utterance or until end of file
      
      utterance_ptr_front = (short int *)malloc(num_samples*SAMPLE_SIZE*number_of_channels);   
      utterance_ptr = utterance_ptr_front;
      samples_read = fread(utterance_ptr, SAMPLE_SIZE, (num_samples*number_of_channels), audio_file);
 
      if(samples_read)
	{
	  if(number_of_channels == 1)
	    {
	      sprintf(output_file_name, CHANNEL_0_FORMAT, output_dir, 
		      base_name, prefix, utterance_number, extension);
	      output_file = fopen(output_file_name, "w");
	      fwrite(utterance_ptr, SAMPLE_SIZE, (unsigned int)samples_read, output_file);
	      fclose(output_file);
	    }
	  else   // copy odd samples to channel_0 and even samples to channel_1
	    {
	      channel_0_front = (short int *)malloc((unsigned int)((num_samples * SAMPLE_SIZE)+SAMPLE_SIZE));
	      channel_1_front = (short int *)malloc((unsigned int)((num_samples * SAMPLE_SIZE)+SAMPLE_SIZE));
	      channel_0 = channel_0_front;
	      channel_1 = channel_1_front;
	      for(counter=0; (counter < (samples_read / number_of_channels));counter++)
		{
		  *channel_0 = *utterance_ptr;
		  utterance_ptr++;
		  channel_0++;
		  *channel_1 = *utterance_ptr;
		  utterance_ptr++;
		  channel_1++;
		}
	      // write to disk each of the channels
	      
	      printf("Writing files:  ");

	      sprintf(output_file_name, CHANNEL_0_FORMAT, output_dir, 
		      base_name, prefix, utterance_number, extension);
	      printf("%s  ", output_file_name);
	      output_file=fopen(output_file_name, "w");
	      (void)fwrite(channel_0_front, SAMPLE_SIZE, counter, output_file);
	      fclose(output_file);
	      
	      sprintf(output_file_name, CHANNEL_1_FORMAT, output_dir,
		      base_name, prefix, utterance_number, extension);
	      printf("%s\n", output_file_name);
	      output_file=fopen(output_file_name, "w");
	      (void)fwrite(channel_1_front, SAMPLE_SIZE, counter, output_file);
	      fclose(output_file);
	      
	      free(channel_0_front);
	      free(channel_1_front);
	    }
	  free(utterance_ptr_front);
	  utterance_ptr = NULL;
	}
      fflush(NULL);
    }
  
// clean up and exit gracefully

  fclose(audio_file);
  utterance_ptr = NULL;
  channel_0_front = NULL;
  channel_1_front = NULL;
  channel_0 = NULL;
  channel_1 = NULL;
  

  printf("Completed make_files.exe with no errors.\n");
  return(ISIP_NO_ERROR);
}

