// file:  TBA

// system include files
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// isip include files
#include <math/integral/integral.h>
#include <math/integral/integral_constants.h>

// local include files
#include "make_files.h"

//
// this is the primary read and write function for make_files
//   it also happens to be the main function for make_files
//
main(int argc, char *argv[])
{

  FILE    *data_file=NULL, *audio_file=NULL, *parameter_file=NULL;
  float_4 beg_time = 0.0, end_time = 0.0, utterance_duration = 0.0;
  int_2   *channel_0_front=NULL, *channel_1_front=NULL, 
          *utterance_ptr_front=NULL;
  int_1   number_of_channels=0;
  int     utterance_number = 0;
  int_2   sample_frequency = 0;
  uint_4  num_samples=0, samples_read=0, utterance_mem_size=0, 
          old_utterance_mem_size=0;
  char    endpointer_data_name[MAX_LINE_LENGTH] = DEFAULT_ENDPOINTER_DATA_NAME,
          param_file_name[MAX_LINE_LENGTH] = DEFAULT_PARAM_FILE_NAME,
          output_dir[MAX_LINE_LENGTH] = DEFAULT_OUTPUT_DIR, 
          base_name[MAX_LINE_LENGTH] = DEFAULT_BASE_NAME, 
          prefix[MAX_LINE_LENGTH] = DEFAULT_PREFIX, 
          extension[MAX_LINE_LENGTH] = DEFAULT_EXTENSION, 
          data_file_string[MAX_LINE_LENGTH] = DEFAULT_DATA_FILE_STRING,
          audio_file_name[MAX_LINE_LENGTH] = DEFAULT_AUDIO_FILE_NAME;

  if(get_command_line(argc, argv, param_file_name, output_dir, base_name, 
		       prefix, extension, audio_file_name, 
		       endpointer_data_name) == ISIP_ERROR)
    {
      fprintf(stderr, "%s: error in command line arguments!\n",
	      ISIP_PROGRAM_NAME);  
      fprintf(stderr, "Execution terminated.\n");
      exit(ISIP_FATAL_ERROR);
    }

  // open and test each of the files
  
  parameter_file = fopen(param_file_name, "r");
  if(!parameter_file)
    {
      fprintf(stderr, "Parameter file (%s) could not be opened.\n", 
	      param_file_name);
      fprintf(stderr, "Execution terminated.\n");
      exit(ISIP_FATAL_ERROR);
    }

  data_file = fopen(endpointer_data_name, "r");
  if(!data_file)
    {
      fprintf(stderr, "Endpoints data file (%s) could not be opened!\n", 
	      endpointer_data_name);
      fprintf(stderr, "Execution terminated.\n");
      exit(ISIP_FATAL_ERROR);
    }

  audio_file = fopen(audio_file_name, "r");
  if(!audio_file)
    {
      fprintf(stderr, "Audio file (%s) could not be opened!\n", 
	      audio_file_name);
      fprintf(stderr, "Execution terminated.\n");
      exit(ISIP_FATAL_ERROR);
    }

  // get parameters from parameter_file
  
  number_of_channels = (int)get_parameters("number_of_channels", 
					   parameter_file);

  sample_frequency = (int)get_parameters("sample_frequency", parameter_file);
  fclose(parameter_file);
  
  // read in an utterance position from data_file, the coorsponding data 
  // from the audio_file, and write the raw audio utterance data to disk.

  while((fgets(data_file_string, sizeof(data_file_string), data_file)))
    {      
      sscanf(data_file_string, SSCANF_FORMAT, &utterance_number, 
	     &beg_time, &end_time);
  
      // move to that position in the file
      
      fseek(audio_file, (long int)(beg_time * sample_frequency * SAMPLE_SIZE * 
				   number_of_channels), SEEK_SET);
      
      // calculate the utterance duration and how many samples that 
      // duration will have
      
      utterance_duration = (end_time) - (beg_time);
      if(utterance_duration <= 0) 
	{
	  fprintf(stderr, 
	       "Illogical utterance duration %5.3f found for utterance %d.  ",
		  utterance_duration, utterance_number);  
	  fprintf(stderr,
		  "Cooresponding utterance file will not be written.\n");
	  utterance_duration = 0;
	}
      num_samples = (long unsigned int)((utterance_duration) *
					(sample_frequency));
      
      // read the whole utterance into memory or until end of file
      
      (unsigned int)utterance_mem_size = (num_samples * SAMPLE_SIZE * 
					  number_of_channels);
      
      // do we need more memory allocated for this utterance than any previous?

      if(utterance_mem_size > old_utterance_mem_size)
	{
	  // reallocate memory
	  
	  utterance_ptr_front = 
	    (int_2 *)realloc(utterance_ptr_front, utterance_mem_size);
	  channel_0_front = (int_2 *)realloc(channel_0_front,
				       (unsigned int)(utterance_mem_size / 2));
	  channel_1_front = (int_2 *)realloc(channel_1_front,
				       (unsigned int)(utterance_mem_size / 2));
	  old_utterance_mem_size = utterance_mem_size;
	}
      samples_read = fread(utterance_ptr_front, SAMPLE_SIZE, 
			   (num_samples*number_of_channels), audio_file);
      if(samples_read)
	{
	  fprintf(stdout, "Writing file(s):  ");
	  
	  if(number_of_channels == 2)
	    {
	      seperate_data(utterance_ptr_front, channel_0_front, 
			    channel_1_front, samples_read);
	      write_data(channel_0_front, channel_1_front, samples_read,
			 number_of_channels, utterance_number,
			 output_dir, base_name, prefix, extension);
	    }
	  else
	    {
	      write_data(utterance_ptr_front, NULL, samples_read,
			 number_of_channels, utterance_number,
			 output_dir, base_name, prefix, extension);
	    }
	}
    }
  
  // clean up and exit gracefully
  
  fclose(audio_file);
  
  free(utterance_ptr_front);
  free(channel_0_front);
  free(channel_1_front);
  
  printf("Completed make_files.exe with no apparent errors.\n");
  return(ISIP_NO_ERROR);
}




