/* File: read_and_debias_data.c */

/*
 * Richard Duncan
 * Institute for Signal and Information Processing
 * December 1995
 *
 * This function counts the number of records, allocates data space in
 * memory for this data, reads in the data, finds the average value of
 * data, debiases it by subtracting the average value from all samples,
 * sets a few fields in the header, and returns a pointer to the data
 *
 * Revision (12/21/95): no longer debiases or saves the average value
 *
 */

#include<stdio.h>

#ifndef __MAIN_H_
#define __MAIN_H_
#include"main.h"
#endif

#ifndef __SP_SPHERE_H_
#include<sp/sphere.h>
#define __SP_SPHERE_H_
#endif

#ifndef __READ_AND_DEBIAS_DATA_H_
#define __READ_AND_DEBIAS_DATA_H_
#include "read_and_debias_data.h"
#endif

char *read_and_debias_data(struct header_t *header,
			   char *data_filename, long *count, long *size){
  
  long int i;

  int temp_error;
  long sample_count, begin, end, sample_n_bytes;

  char *c;
  INT_2 *temp, *data;
  
  SP_REAL average, total;

  long min_value, max_value;
  
  FILE *input_fp, *output_fp;

  /*
   * set the sample_n_bytes
   *
   */

  sample_n_bytes = sizeof(INT_2);
  
  /*
   * open the sphere file for reading
   *
   */

  if (( input_fp = fopen(data_filename, "r")) == NULL){
    fprintf(stderr, file_error, "raw data", data_filename);
    exit(31);
  }

  /*
   * determine the number of samples in the file
   *
   */
  rewind(input_fp);
  begin = ftell(input_fp);
  fseek(input_fp, 0, SEEK_END);
  end = ftell(input_fp);
  sample_count = (long int) ((end - begin) / sample_n_bytes);
  rewind(input_fp);

  *count = sample_count;
  *size = sample_n_bytes;
  
  /*
   * allocate memory for the data
   *
   */

  data = (INT_2 *) malloc(sample_count * sample_n_bytes);
  if (data == (INT_2 *)NULL){
    fprintf(stderr, "Error allocating data, exiting\n");
    exit(39);
  }

  /*
   * read in the data from the input file
   *
   */

  if (fread((void *) data, sample_n_bytes, sample_count, input_fp) !=
      sample_count){
    fprintf(stderr,"Error reading data, exiting!\n");
    exit(44);
  }
  
  /*
   * close the input file
   *
   */

  fclose(input_fp);
  
  /*
   * Find the average of the data
   *
   *
   temp = data;
   for(i=0;i<sample_count;i++){
   total += (SP_REAL) abs(*temp);
   temp ++;
   }
   average = (SP_REAL) (total / sample_count);
   */
  
  /*
   * Subtract the average from each sample
   *
   *
   temp = data;
   for(i=0;i<sample_count;i++){
   *temp -= ((INT_2) average);
   temp ++;
   }
   */
  
  /*
   * Find the minimum and maximum values
   *
   */

  temp = data;
  min_value = *temp;
  max_value = *temp;
  for(i=0; i< sample_count; i++) {
    if (*temp < min_value)
      min_value = *temp;
    if (*temp > max_value)
      max_value = *temp;
    temp++;
  }

  printf("\tmin = %ld, max = %ld\n",
	 min_value,max_value);
   
  /* program no longer saves the average value
     temp_error = sp_add_field(header, "sample_average",
     T_REAL, (char *) &average);
     if (temp_error >= 0)
     */
  temp_error = sp_add_field(header, "sample_min",
			    T_INTEGER, (char *) &min_value);
  if (temp_error >= 0)
    temp_error = sp_add_field(header, "sample_max",
			      T_INTEGER, (char *) &max_value);
  if (temp_error >= 0)
    temp_error = sp_add_field(header, "sample_count",
			      T_INTEGER, (char *) &sample_count);
  if (temp_error < 0) {
    fprintf(stderr,"Error adding crucial fields to header, exiting!\n");
    exit(22);
  }

  
  return((char *)data);
}





