// file: /u1/picone/jeida/plot_signal/get_params.cc
//
// a simple command line parser
//

// system include files
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// isip include files
//
#include <math/integral/integral.h>
#include <math/integral/integral_constants.h>

// local include files
//
#include "plot_signal.h"

// function: get_parameters_C
//
// arguments:
//
//  int argc: argc from the main program
//  char** argv: argv from the main program
//  float_4& sample_freq: sample frequency in hz
//  float_4& start_time: start time in seconds
//  float_4& end_time: end time in seconds
//  int_4& resolution: resolution in points (plotted)
//  int_4& num_bytes: number of bytes per sample
//  int_4& num_chan: number of channels per sample
//  int_4& chan: channel number
//  char_1* file: name of file to be plotted
//
// return value: a logical_1 indicating status
//
// this function parses command line arguments
//
logical_1 get_parameters_C(int argc, char** argv,
			   float_4& sample_freq, float_4& start_time,
			   float_4& end_time, int_4& resolution,
			   int_4& num_bytes, int_4& num_chan, int_4& chan,
			   char_1* file) {
  // pick off the filename
  //
  strcpy((char*)file, argv[argc-1]);

  // main decoding loop
  //
  int_4 last_index = argc - (int_4)1;

  for (int_4 index=1; index<last_index; index++) {

    // pick off sample frequency
    //
    if (strcmp(ISIP_OPT_SF, argv[index]) == (int)0) {
      sample_freq = atof(argv[index+(int)1]);
    }

    // pick off start time
    //
    else if (strcmp(ISIP_OPT_START, argv[index]) == (int)0) {
      start_time = atof(argv[index+(int)1]);
    }

    // pick off end time
    //
    else if (strcmp(ISIP_OPT_END, argv[index]) == (int)0) {
      end_time = atof(argv[index+(int)1]);
    }

    // pick off resolution
    //
    else if (strcmp(ISIP_OPT_RES, argv[index]) == (int)0) {
      resolution = atoi(argv[index+(int)1]);
    }

    // pick off number of bytes
    //
    else if (strcmp(ISIP_OPT_NBYTES, argv[index]) == (int)0) {
      num_bytes = atoi(argv[index+(int)1]);
    }

    // pick off number of channels
    //
    else if (strcmp(ISIP_OPT_NCHAN, argv[index]) == (int)0) {
      num_chan = atoi(argv[index+(int)1]);
    }

    // pick off channel number
    //
    else if (strcmp(ISIP_OPT_CHAN, argv[index]) == (int)0) {
      chan = atoi(argv[index+(int)1]);
    }
  }

  // exit gracefully
  //
  return ISIP_TRUE;
}


