// file: endpoint_calculator.cc
 
#include "endpointer.h"

// external functions prototypes

void energy_2(FILE *, int, float *, int, struct parameter_data *);
void noise_2(float *, int, float *, float *, struct parameter_data *);
void signal_2(float *, int, float *, struct parameter_data *);
int find_critical_points(float *, float *, int, struct critical_points_struct *, int, struct parameter_data *);

// main function

float endpoint_calculator(FILE *source_file, FILE *output_file, struct parameter_data *parameter)
{
  
  // variable declarations

  int file_length;
  int samples_in_frame;
  float *energy_array, *noise_array, *signal_array;
  int number_of_frames;
  int counter;
  float maybe_utterance_start;
  float utterance_start;
  float utterance_end;
  float maybe_utterance_end;
  int utterance_number;
  int critical_points_max;
  int number_of_critical_points;
  struct critical_points_struct *critical_point;


  // initialize some values

  samples_in_frame = (int)(parameter->frame_duration*parameter->sample_frequency);
  critical_points_max = 100000;
 
  // get file_length

  rewind(source_file);
  fseek(source_file,0L,SEEK_END);
  file_length = ftell(source_file);
  rewind(source_file);


  // allocate space for energy_array;
  
  number_of_frames = (int)((file_length / (parameter->number_of_channels * SAMPLE_SIZE * samples_in_frame)+1));
  energy_array = (float *)malloc(sizeof(float) * number_of_frames);
  signal_array = (float *)malloc(sizeof(float) * number_of_frames);
  noise_array  = (float *)malloc(sizeof(float) * number_of_frames);
  critical_point = (struct critical_points_struct *)malloc(sizeof(struct critical_points_struct) *critical_points_max);


// calculate energy, noise, signal, and critical points

  energy_2(source_file, file_length, energy_array, number_of_frames, parameter);
  printf("Energy levels calculated\n");
  signal_2(energy_array, number_of_frames, signal_array, parameter);
  printf("Signal levels calculated\n");
  noise_2(energy_array, number_of_frames, noise_array, signal_array, parameter);
  printf("Noise levels calculated\n");
  number_of_critical_points = find_critical_points(signal_array, noise_array, number_of_frames, critical_point, critical_points_max, parameter);
  printf("Critical points found\n");


// debugging stuff - prints out critical points
/*  
  printf("number of critical points = %d\n", number_of_critical_points);
counter = 0;
  while(counter < number_of_critical_points)
  {
    printf("critical point #%3d found at time %5.2f with slope %d\n", counter, (critical_point + counter)->time, (critical_point + counter)->positive_slope);
    counter ++;
  }
*/
// end of debugging stuff



// debugging stuff - prints out energy, noise, and signal
/*
  counter = 0;
  while (counter < number_of_frames)
    {
     printf("%5.2f %5.2f %5.2f %5.2f\n", (float)counter * parameter->frame_duration, energy_array[counter], noise_array[counter], signal_array[counter]);
      counter ++; 
    }
*/
// end of debugging stuff


utterance_start = 0;
utterance_end  = 0;
maybe_utterance_start = 0;
maybe_utterance_end = 0;
counter=0;
utterance_number =0;

while (counter <  number_of_critical_points)
  {
//    printf("maybe_utterance_start: %5.2f %d\n",critical_point->time, critical_point->positive_slope);

    maybe_utterance_start = critical_point->time;
    while(((critical_point+2)->time - (critical_point+1)->time) < parameter->minimum_utterance_separation && (counter < number_of_critical_points))
      {
//	 printf("in loop critical point: %5.2f %d\n",critical_point->time, critical_point->positive_slope);

 	critical_point += 2;
	counter += 2;
	if(counter +2 > number_of_critical_points)
	  {
//	    printf("break\n");
	    critical_point -= 2;
	    break;
	  }
      }
    maybe_utterance_end = (critical_point + 1)->time;

//    printf("maybe_utterance_end: %5.2f %d\n",(critical_point+1)->time, (critical_point+1)->positive_slope);

    if((maybe_utterance_end - maybe_utterance_start) > parameter->minimum_utterance_duration)
      {

// rejoice! utterance has been found!

	utterance_start = maybe_utterance_start - parameter->initial_padtime;
	utterance_end = maybe_utterance_end + parameter->final_padtime;

// make sure utterance doesn't start before t=0;

       if(utterance_start < 0)
	 {
	   utterance_start = 0;
	   fprintf(stderr,"Warning: file may start with an utterance in progress\n");
	 }

// make sure utterance doesn't end after end of file
	
	if(utterance_end > number_of_frames * parameter->frame_duration)
	  {  
	    utterance_end = number_of_frames * parameter->frame_duration;
	    fprintf(stderr,"Warning: file may end with an utterance in progress\n");
	  }

// output endpoints to file
	
	fprintf(output_file, "%4d beg_time = %7.2f secs     end_time = %7.2f secs\n",utterance_number, utterance_start, utterance_end);
	utterance_number ++;
      }
    counter +=2;
    critical_point +=2;
    
  }
  
printf("Endpoints found\n");
      
return(0);

}




























































































































