# file: $JEIDA/src/validate/v1.0/src/display.tcl
#
# This program acts as the gui for the validator
#

# declare all the globals
#
global val_file_name
global transcription
global old_transcription
global comments
global old_comments
global audio_command
global save_flag
global maxcolnum
global env
global validator_title
global p
global label_transcription
global audio_device_label
global raw_filename_label
global filename_label
global wordlist_label
global directory_label
global last_date_label
global last_validator_label
global amplitude length
global label_transcription_saved label_transcription_edited
global tmp_file_name
global ISIP_VALHOME
# initialize the variables
#
set transcription ""
set comments ""

# withdraw the existing toplevel, to make way for new one
#
wm withdraw .

# destroy the toplevel, if neccessary
#
catch {destroy $p}

# set up the new toplevel
#
toplevel $p

# no size set for $p yet, packing will be used
#
wm title $p $validator_title

# set up the frames
#
frame $p.top_frame 
frame $p.middle_frame 
frame $p.plot_frame 
frame $p.topleft_frame -borderwidth 2 -background black
frame $p.button_frame 
frame $p.listbox_frame
frame $p.button_lower 
frame $p.button_upper
frame $p.transcription_frame -borderwidth 2 -background black
frame $p.comments_frame -borderwidth 2 -background black
frame $p.old_transcription_frame -borderwidth 2 -background black
frame $p.old_comments_frame -borderwidth 2 -background black
frame $p.wordlist_frame

# set up the quit button
#
button $p.quit -text "Quit Validator" -command {
    if {$save_flag != 0} {
	#puts $transcription
	save_file
	catch { exec rm $tmp_file_name}
    }
    exit
} -borderwidth 5 -height 2


# set up the audio device label
#
label $p.audio_device -textvariable audio_device_label \
	-relief raised -anchor w

# set up the raw filename label
#
label $p.raw_filename -textvariable raw_filename_label -relief raised \
	-anchor w -height 2

# set up the filename label
#
label $p.filename -textvariable filename_label -relief raised -anchor w \
	-height 2

# set up the wordlist label
#
label $p.wordlist -textvariable wordlist_label -relief raised -anchor w

# set up the select_wordlist button
#
button $p.select_wordlist -bitmap "@$ISIP_VALHOME/decr" -command select_wordlist

# set up the directory label
#
label $p.directory -textvariable directory_label -relief raised -anchor w

# set up the last date label
#
label $p.last_date -textvariable last_date_label -relief raised -width 42 \
	-anchor w

# set up the last validator label
#
label $p.last_validator -textvariable last_validator_label \
	-relief raised -anchor w

# set up the transcription entry widget and label
#
label $p.label_transcription -textvariable label_transcription \
	-relief raised -height 2 -width 25 -anchor w
entry $p.transcription -textvariable transcription -relief raised 

# bind a keypress in the transcription area to set the save flag
#
bind $p.transcription <ButtonRelease-1> { 
    set save_flag 1 
    set label_transcription $label_transcription_edited
    update
}

# bind escape key to save/next/play in the transcription area, since it will
# have the input focus at almost all times
#
bind $p.transcription <Return> {
    save_next_play
}

bind $p.transcription <Escape> {
    save_next_play
}

# set up the comments entry widget and label
#
label $p.comments_label -text " Enter Comments:" -relief raised \
	-height 2 -width 25 -anchor w
entry $p.comments -textvariable comments -relief raised 


# bind a keypress in the comments area to set the save flag
#
bind $p.comments <ButtonRelease-1> { 
    set label_transcription $label_transcription_edited
    update
    set save_flag 1 
}

bind $p.comments <KeyRelease> {
    if {$save_flag != 1} {
	set save_flag 1
	set label_transcription $label_transcription_edited
	update
    }
}

# bind escape key to save/next/play in the comments area, since it will
# have the input focus a few times
#
bind $p.comments <Return> {
    save_next_play
}

bind $p.comments <Escape> {
    save_next_play
}

# set up the old transcription label widget and label
#
label $p.old_label_transcription -text " Previous Transcription:" \
	-relief raised -height 2 -width 25 -anchor w 
label $p.old_transcription -text old_transcription -relief raised \
	-height 2  -anchor w 


# set up the old comment label widget and label
#
label $p.old_comments_label -text " Previous Comment:" -relief raised \
	-height 2 -width 25 -anchor w
label $p.old_comments -text old_comments -relief raised -height 2 \
	-anchor w 

# set up the Play transcription button
#
button $p.play -text "Play" -command {
    proc_play
}  -borderwidth 5 -height 3

# set up the Next transcription button
#
button $p.next -text "Next" -command {
    proc_file next
    proc_plot
    update_window
} -borderwidth 5

# set up the Previous transcription button
#
button $p.prev -text "Prev" -command {
    proc_file prev
    proc_plot
    update_window
}  -borderwidth 5

# set up the Save transcription button
#
button $p.save -text "Save" -command {
    if { $save_flag == 0 } { puts "\7" }
    save_file
} -borderwidth 5

# set up the Load file button
#
button $p.load_file -text "Load" -command {
    save_file
    display_load_file
    update_window
    proc_plot
} -borderwidth 5

# set up the clear button
#
button $p.clear -text "Clear" -command {
    set transcription ""
    set comments ""
    set save_flag 1
    set label_transcription $label_transcription_edited
    update
}  -borderwidth 5

# set up the Save, Play, and Next button
#
button $p.save_next_play -text "Save, Next, Play" -command {
    save_next_play
} -height 3 -borderwidth 5

# bind the right mouse button in the save play next button to enter
# garbage for transcription
#
bind $p.save_next_play <Button-3> {
    set transcription "<garbage>"
    set comments ""
    set save_flag 1
    set label_transcription $label_transcription_edited
    update
}


# set up the canvas for drawing the plot
#
canvas $p.plot

# set up the amplitude and length markers for the plot
#
label $p.plot.amplitude -text plot_amplitude
label $p.plot.length -text plot_length

# pack the frames into $p
#

pack $p.top_frame $p.middle_frame $p.plot_frame -side top -fill both
pack $p.topleft_frame $p.listbox_frame -side left -fill both \
	-in $p.top_frame -expand true

# pack the top left frame
#
pack $p.quit -in $p.topleft_frame -fill both
pack $p.audio_device $p.raw_filename $p.filename $p.wordlist_frame \
	$p.directory $p.last_date $p.last_validator $p.button_frame \
	-fill both -in $p.topleft_frame 

# pack the button frame
#
pack $p.button_upper $p.button_lower -in $p.button_frame \
	-side top -fill both
pack $p.load_file $p.prev $p.save_next_play -side left \
	-in $p.button_upper -fill both -expand true
pack $p.save $p.next $p.play $p.clear -side left \
	-in $p.button_lower -fill both -expand true

# pack the wordlist frame
#
pack $p.wordlist $p.select_wordlist -in $p.wordlist_frame -side left\
	-fill both -expand true

# pack the middle frame
#
$p.middle_frame configure -relief raised 
pack $p.transcription_frame $p.comments_frame $p.old_transcription_frame \
	$p.old_comments_frame -fill both -in $p.middle_frame -side top
pack $p.old_label_transcription\
	-in $p.old_transcription_frame -fill both -side left
pack $p.old_transcription -in $p.old_transcription_frame -fill both \
	-expand true
pack $p.old_comments_label -in $p.old_comments_frame -side left -fill both
pack $p.old_comments -in $p.old_comments_frame -side left -fill both \
	-expand true
pack $p.label_transcription -side left \
	-fill y -in $p.transcription_frame
pack $p.transcription -in $p.transcription_frame -side left \
	-fill both -expand true
pack $p.comments_label -in $p.comments_frame -side left\
	-fill both
pack $p.comments -in $p.comments_frame -side left\
	-fill both -expand true


# pack the canvas frame to the bottom
#
$p.plot configure -width $width_of_plot -height $height_of_plot
pack $p.plot -in $p.plot_frame

# place the ampliture and time length tags into the canvas
#

place $p.plot.amplitude -x 0 -y 0
place $p.plot.length -y 180 -x -0

# pack all the listbox frames into $p.listbox
#
for {set colnum 0} {$colnum < $maxcolnum} {incr colnum} {
    frame $p.word_frame$colnum
    pack $p.word_frame$colnum -in $p.listbox_frame -side left -fill both \
	    -expand true
}

# set up the multiple word listboxes
#
for {set colnum 0} {$colnum < $maxcolnum} {incr colnum} {
    listbox $p.word$colnum -background gray80 \
	    -font -*-fixed-bold-r-normal--13-* -relief raised
    pack $p.word$colnum -in $p.word_frame$colnum -fill both -expand true
    
    # set up the bindings for each listbox
    #
    bind $p.word$colnum <ButtonRelease-1> {
	set word_trimming [selection get]

	# trim the last two characters from selection string
	#
	set word_length [string length $word_trimming]
	incr word_length -2
	set word_trimming [string range $word_trimming 0 $word_length]
	set transcription "$transcription $word_trimming"

	# trim off excess spaces
	#
	set transcription [string trim $transcription]
	
	# set the save flag, and make visible the "change astrick"
	#
	set save_flag 1
	set label_transcription $label_transcription_edited
	update
    }

    # bind save_next_play to button 2
    #
    bind $p.word$colnum <Button-2> {
	save_next_play
    }

    # bind delete_last_word to button 3
    #
    bind $p.word$colnum <Button-3> { 
	delete_last_word 
    }

}











